﻿using DotNetCore.CAP;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.FlowEngine.Workflows;
using Hotline.Orders;
using Hotline.Settings.TimeLimits;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Order;
using MapsterMapper;
using MediatR;
using Microsoft.Extensions.Logging;

namespace Hotline.Application.Handlers.FlowEngine
{
    public class WorkflowPreviousHandler : INotificationHandler<PreviousNotify>
    {
        private readonly IOrderDomainService _orderDomainService;
        private readonly IOrderRepository _orderRepository;
        private readonly IWorkflowDomainService _workflowDomainService;
        private readonly ITimeLimitDomainService _timeLimitDomainService;
        private readonly ICapPublisher _capPublisher;
        private readonly IMapper _mapper;
        private readonly ILogger<WorkflowPreviousHandler> _logger;

        public WorkflowPreviousHandler(
            IOrderDomainService orderDomainService,
            IOrderRepository orderRepository,
            IWorkflowDomainService workflowDomainService,
            ITimeLimitDomainService timeLimitDomainService,
            ICapPublisher capPublisher,
            IMapper mapper,
            ILogger<WorkflowPreviousHandler> logger)
        {
            _orderDomainService = orderDomainService;
            _orderRepository = orderRepository;
            _workflowDomainService = workflowDomainService;
            _timeLimitDomainService = timeLimitDomainService;
            _capPublisher = capPublisher;
            _mapper = mapper;
            _logger = logger;
        }

        /// <summary>Handles a notification</summary>
        /// <param name="notification">The notification</param>
        /// <param name="cancellationToken">Cancellation token</param>
        public async Task Handle(PreviousNotify notification, CancellationToken cancellationToken)
        {
            var workflow = notification.Workflow;
            var data = notification.Dto;

            switch (workflow.ModuleCode)
            {
                case WorkflowModuleConsts.OrderHandle:
                    var order = await _orderDomainService.GetOrderAsync(workflow.ExternalId, true, true,
                        cancellationToken);
                    order.CheckIfFiled();
                    _mapper.Map(workflow, order);
                    if (notification.TargetStep.StepType is EStepType.Start)
                    {
                        order.Status = EOrderStatus.BackToUnAccept;
                        order.BackToUnsign();
                    }

                    await _orderRepository.UpdateAsync(order, cancellationToken);
                    if (notification.IsOrgToCenter)
                    {
                        var expiredTimeConfig = _timeLimitDomainService.CalcExpiredTime(DateTime.Now, EFlowDirection.OrgToCenter, order.AcceptTypeCode);

                        await _workflowDomainService.UpdateExpiredTimeAsync(workflow, expiredTimeConfig.ExpiredTime,
                            expiredTimeConfig.TimeText, expiredTimeConfig.Count, expiredTimeConfig.TimeType,expiredTimeConfig.NearlyExpiredTime, cancellationToken);

                        var dto = _mapper.Map<OrderDto>(order);
                        await _capPublisher.PublishAsync(Hotline.Share.Mq.EventNames.HotlineOrderExpiredTimeUpdate, dto, cancellationToken: cancellationToken);
                        //try
                        //{
                        //    await _provinceService.HotlineOrderFlowRecalled(dto, cancellationToken);
                        //}
                        //catch (Exception e)
                        //{
                        //    _logger.LogError("_provinceService.HotlineOrderFlowRecalled throw exception: {ex}",
                        //        e.Message);
                        //}
                    }

                    break;
                case WorkflowModuleConsts.KnowledgeAdd:
                case WorkflowModuleConsts.KnowledgeUpdate:
                case WorkflowModuleConsts.KnowledgeDelete:
                case WorkflowModuleConsts.TelRestApply:
                    break;
            }
        }
    }
}