﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Application.FlowEngine;
using Hotline.DataSharing.Province.Notifications;
using Hotline.File;
using Hotline.Orders;
using Hotline.Share.Dtos.File;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using MapsterMapper;
using MediatR;
using Microsoft.AspNetCore.Http;
using XF.Domain.Exceptions;

namespace Hotline.Application.Handlers.Order
{
    public class ReceiveOrderNotifyHandler : IRequestHandler<ReceiveOrderNotify, AddOrderResponse?>
    {
        private readonly IOrderRepository _orderRepository;
        private readonly IOrderDomainService _orderDomainService;
        private readonly IWorkflowApplication _workflowApplication;
        private readonly IMapper _mapper;
        private readonly IFileRepository _fileRepository;

		public ReceiveOrderNotifyHandler(
            IOrderRepository orderRepository,
            IOrderDomainService orderDomainService,
            IWorkflowApplication workflowApplication,
            IMapper mapper,
            IFileRepository fileRepository)
        {
            _orderRepository = orderRepository;
            _orderDomainService = orderDomainService;
            _workflowApplication = workflowApplication;
            _mapper = mapper;
            _fileRepository = fileRepository;
		}

        /// <summary>Handles a request</summary>
        /// <param name="request">The request</param>
        /// <param name="cancellationToken">Cancellation token</param>
        /// <returns>Response from the request</returns>
        public Task<AddOrderResponse?> Handle(ReceiveOrderNotify request, CancellationToken cancellationToken)
        {
            var dto = request.AddOrderDto;
            var files = request.Files;
            switch (dto.Source)
            {
                case ESource.ProvinceStraight:
                    return ReceiveOrderFromProvinceAsync(dto, files, cancellationToken);
                case ESource.Police110:
                case ESource.CityDataExchangeLz:
                case ESource.ConvergenceMedia:
                case ESource.WebPortal:
                    return ReceiveOrderFromOtherPlatformAsync(dto, files, cancellationToken);
                case ESource.Hotline:
                case ESource.HotlineImport:
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }

        /// <summary>
        /// 接受外部工单（除省平台）
        /// </summary>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        private async Task<AddOrderResponse?> ReceiveOrderFromOtherPlatformAsync(AddOrderDto dto, List<FileDto> files,

			CancellationToken cancellationToken)
        {
            if (string.IsNullOrEmpty(dto.ExternalId))
                throw new UserFriendlyException("工单外部编号不能为空");

            var exists = await _orderRepository.Queryable().Where(d => d.ExternalId == dto.ExternalId).FirstAsync(cancellationToken);
            if (exists != null)
            {
	            if (files != null && files.Any()) exists.FileJson = await _fileRepository.AddFileAsync(files, exists.Id, "",cancellationToken);
	            await _orderRepository.FileAsync(exists, cancellationToken);
				return null;
            }
            else
            {
				var order = _mapper.Map<Hotline.Orders.Order>(dto);
                order.InitId();
				if (files != null && files.Any()) order.FileJson = await _fileRepository.AddFileAsync(files, order.Id, "", cancellationToken);
				var orderId = await _orderDomainService.AddAsync(order, cancellationToken);
				return new AddOrderResponse
				{
					Id = orderId,
					No = order.No,
					Password = order.Password!
				};
			}
        }

        /// <summary>
        /// 接受省平台工单
        /// </summary>
        private async Task<AddOrderResponse?> ReceiveOrderFromProvinceAsync(AddOrderDto dto, List<FileDto> files, CancellationToken cancellationToken)
        {
            if (string.IsNullOrEmpty(dto.ProvinceNo))
                throw new UserFriendlyException("无效省工单编号");

            var order = await _orderRepository.GetAsync(d => d.ProvinceNo == dto.ProvinceNo, cancellationToken);
            if (order is null)
            {
                order = _mapper.Map<Hotline.Orders.Order>(dto);
                order.InitId();
                if (files != null && files.Any()) order.FileJson = await _fileRepository.AddFileAsync(files, order.Id, "", cancellationToken);
				await _orderDomainService.AddAsync(order, cancellationToken);

                if (order.Source is ESource.ProvinceStraight)
                {
                    var orderExtension = await _orderDomainService.GetOrderExtensionsAsync(dto.ProvinceNo, cancellationToken);
                    if (orderExtension is not null)
                    {
                        orderExtension.Id = order.Id;
                        await _orderDomainService.UpdateExtensionAsync(orderExtension, cancellationToken);
                    }
                }
				return new AddOrderResponse
                {
                    Id = order.Id,
                    No = order.No,
                    Password = order.Password!
                };
            }
            else
            {
	            if (files != null && files.Any()) order.FileJson = await _fileRepository.AddFileAsync(files, order.Id, "", cancellationToken);
	            await _orderRepository.FileAsync(order, cancellationToken);
				// 特提（撤回至发起）
				if (!string.IsNullOrEmpty(order.WorkflowId))
                    await _workflowApplication.RecallToStartAsync(order.WorkflowId, "省工单重派", cancellationToken);
                return _mapper.Map<AddOrderResponse>(order);
            }
        }
    }
}