﻿using Hotline.FlowEngine.Definitions;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.Settings;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Settings;
using Hotline.Users;
using XF.Domain.Entities;

namespace Hotline.FlowEngine.Workflows
{
    public interface IWorkflowDomainService
    {
        /// <summary>
        /// 创建流程
        /// </summary>
        Task<Workflow> CreateWorkflowAsync(WorkflowModule wfModule, string title, string userId, string userCode,
            string? externalId = null, string? timelimitText = null, int? timelimitCount = null, ETimeType? timeType = null, DateTime? expiredTime = null, DateTime? nearlyExpiredTime = null,
            CancellationToken cancellationToken = default);

        /// <summary>
        /// 进行流程的开始节点
        /// </summary>
        Task StartAsync(Workflow workflow, WorkflowStep startStep, BasicWorkflowDto dto, StepDefine firstStepDefine,
            FlowAssignInfo flowAssignInfo, CancellationToken cancellationToken);

        /// <summary>
        /// 查询工作流
        /// </summary>
        Task<Workflow> GetWorkflowAsync(string workflowId, bool withDefine = false, bool withSteps = false,
            bool withTraces = false, bool withSupplements = false, bool withCountersigns = false,
            CancellationToken cancellationToken = default);

        /// <summary>
        /// 查询工作流包含当前用户办理权限（是否可办理）
        /// </summary>
        Task<(Workflow, bool, bool)> GetWorkflowHandlePermissionAsync(string workflowId, string userId, string orgCode,
            CancellationToken cancellationToken = default);

        /// <summary>
        /// 受理,接办
        /// </summary>
        Task AcceptAsync(Workflow workflow, string userId, string? userName, string orgId, string? orgName, string? orgAreaCode,
            string? orgAreaName, CancellationToken cancellationToken);

        /// <summary>
        /// 办理（流转至下一节点）
        /// </summary>
        Task NextAsync(Workflow workflow, WorkflowStep currentStep, NextWorkflowDto dto, StepDefine nextStepDefine,
            FlowAssignInfo flowAssignInfo, CancellationToken cancellationToken);

		/// <summary>
		/// 退回（返回前一节点）
		/// </summary>
		/// <returns></returns>
        Task PreviousAsync(Workflow workflow, PreviousWorkflowDto dto, User user, CancellationToken cancellationToken);

        /// <summary>
        /// 撤回（返回到之前任意节点）
        /// </summary>
        Task RecallAsync(Workflow workflow, RecallDto dto, StepDefine targetStepDefine, FlowAssignInfo flowAssignInfo,
            CancellationToken cancellationToken);

        /// <summary>
        /// 跳转（直接将流程跳转至任意节点）
        /// </summary>
        Task JumpAsync(Workflow workflow, RecallDto dto, StepDefine targetStepDefine, FlowAssignInfo flowAssignInfo,
            CancellationToken cancellationToken);

        /// <summary>
        /// 重办
        /// </summary>
        Task RedoAsync(Workflow workflow, RecallDto dto, StepDefine targetStepDefine, FlowAssignInfo flowAssignInfo,
            CancellationToken cancellationToken);

        /// <summary>
        /// 否决（审批流程不通过）
        /// </summary>
        /// <returns></returns>
        Task RejectAsync(Workflow workflow, BasicWorkflowDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 补充
        /// </summary>
        /// <returns></returns>
        Task SupplementAsync(Workflow workflow, EndWorkflowDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 终止流程
        /// </summary>
        Task TerminateAsync(TerminateDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 结束流程（流程直接流转至结束节点）
        /// </summary>
        Task<WorkflowTrace> EndAsync(Workflow workflow, BasicWorkflowDto dto, StepDefine endStepDefine,
            WorkflowStep currentStep, EReviewResult? reviewResult = EReviewResult.Unknown,
            CancellationToken cancellationToken = default);

        StepDefine GetStepDefine(WorkflowDefinition workflowDefinition, string stepCode);

        /// <summary>
        /// 查询当前待办理节点
        /// </summary>
        WorkflowStep FindCurrentStepWaitForHandle(Workflow workflow);

        /// <summary>
        /// 查询待回访部门
        /// </summary>
        Task<(Kv, IReadOnlyList<Kv>)> GetUnvisitOrgsAsync(string workflowId, CancellationToken cancellationToken);

        /// <summary>
        /// 依据配置过滤下一节点
        /// </summary>
        List<StepDefine> NextStepDefineFilter(EPathPolicy pathPolicy, List<StepDefine> nextStepDefines);

        /// <summary>
        /// 撤销流程
        /// </summary>
        Task CancelAsync(CancelDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 更新期满时间
        /// </summary>
        Task UpdateExpiredTimeAsync(Workflow workflow, DateTime expiredTime, string timelimit, int? timelimiteCount,
            ETimeType? timelimitUnit, DateTime nearlyExpiredTime, CancellationToken cancellationToken);

        /// <summary>
        /// 新增流程流转记录
        /// </summary>
        Task AddTracesAsync(string workflowId, List<WorkflowTrace> traces, CancellationToken cancellationToken);

        /// <summary>
        /// 创建开始节点
        /// </summary>
        WorkflowStep CreateStartStep(Workflow workflow, StepDefine startStepDefine, BasicWorkflowDto dto, List<Kv> handlers);

        /// <summary>
        /// 结束会签(会签未办理节点无需再办，由发起会签节点继续办理)
        /// </summary>
        Task EndCountersignAsync(string workflowId, CancellationToken cancellationToken);

		/// <summary>
		/// 查询未完成节点
		/// </summary>
		/// <param name="steps"></param>
		/// <param name="orgCode"></param>
		/// <param name="userId"></param>
		/// <returns></returns>
		WorkflowStep GetUnHandleStep(List<WorkflowStep> steps, string orgCode, string userId);

	}
}